function [T_EE,J]=nR_MGD(theta)
% 
% Rsolution du problme gomtrique direct (MGD) pour un manipulateur 
% sriel  n articulations rotodes (nR)
%
% Exemples d'appel de la commande :
%   [T_EE,J] = nR_MGD([0, ... ,0])
%   [T_EE,J] = nR_MGD()
%
% - Sans argument, la fonction retourne les coordonnes homognes (T_EE) et
%   la matrice Jacobienne (J) sous forme symbolique.
% - Avec [theta_1,...,theta_n] comme argument, la fonction retourne les
%   coordonnes homognes (T_EE) et la matrice Jacobienne (J) sous forme
%   numrique
%
% !!! IMPORTANT : modifier les valeurs de 'theta', 'a', 'b', 'alpha' ci-bas
%     pour rflter les valeurs de votre manipulateur (lignes 33-36 et 50-52)
%
% Note : 'help nR_MGD' tap  la ligne de commande affiche ces instructions


%----------------------------------------
% Dfinitions des constantes (!  MODIFIER)
%----------------------------------------

    % Prcision arithmtique dsire pour Maple (Affecte la commande 'vpa')
    digits(8);

    %------------------------
    % Si aucun argument (nargin == 0), retourne la jacobienne symbolique
    % o 't1' ... 'tn' reprsentent symboliquement les 'theta 1'  'theta n'
    if (nargin == 0)
        theta = [sym('t1','real'),...  % ! AJOUTER OU SUPPRIMER CES LIGNES
                 sym('t2','real'),...  % AU BESOIN POUR QU'IL Y AIT AUTANT
                 sym('t3','real'),...  % DE LIGNES QUE D'ARTICULATIONS
                 sym('t4','real')];
    end

    %------------------------
    % Paramtres de Denavit-Hartenberg du robot (Voir Sec.:4.2)
    % !  MODIFIER
    
    % Robot FANUC ARC MATE 120iL (joint 1  6 et outil)
%     a     = [  0.200,  0.800, -0.070,  0,      0,     0,      0         ];
%     b     = [  0.700,  0,      0,      0.810,  0,     0.100,  0.200     ];
%     alpha = [ -pi/2,   0,      pi/2,  -pi/2,   pi/2,  0,      60/180*pi ];
 
    % Robot 4R sphrique : Positio
    % (test : [T_EE,J] = nR_MGD([3*pi/4, pi/2, 3*pi/4, 3*pi/4] )
    a     = [      0;      0;  1; 2^0.5/2 ];
    b     = [      1;      0;  0;       0 ];
    alpha = [ (pi/2); (pi/2);  0;       0 ];
    
  
    
    
%----------------------------------------
% Corps de la fonction 'nR_MGD'
%----------------------------------------
       
    %------------------------
    % Calcul de A = [e_1 ... e_n] (Voir Eq.:5.7a p.170)
    A = ei(1);
    if length(theta)>1
        for u = 2:length(theta)
            A = [A,ei(u)];
        end
    end

    %------------------------
    % Calcul de B = [e_1 x r_1 ... e_n x r_n] (Voir Eq.:5.7b p.170)
    B = ei_ri(1);
    if length(theta)>1
        for u = 2:length(theta)
            B = [B,ei_ri(u)];
        end
    end

    %------------------------
    % Calcul de la matrice Jacobienne J = [A;B] (Voir Eq.:5.10a et b)
    J = [[A];[B]];
    J = simplify(vpa(J));
    
    if not (nargin == 0)
        J = eval(J);
    end
    
    %------------------------
    % Calcul de la position et de l'orientation (coordonnes homognes) du
    % dernier joint, soit celui du bout de l'outil (End-Effector) si n+1 joints
    T_EE = T(length(theta));
    T_EE = simplify(vpa(T_EE));
    
    if not (nargin == 0)
        T_EE = eval(T_EE);
    end
    
    
    
    
    
    
    
%----------------------------------------
% Dfinitions des fonctions
%----------------------------------------

    %------------------------
    % Fonction mui(i) %Voir Eq.:4.1a
    function val = mui(i)
        val = sin(alpha(i));
    end

    %------------------------
    % Fonction lambdai(i) %Voir Eq.:4.1a
    function val = lambdai(i)
        val = cos(alpha(i));
    end
        
    %------------------------
    % Fonction Q(i) : Permet de construire la matrice de rotation Qi
    % Voir Eq.:4.1e et Eq.:4.7
    function val = Qi(i)
        val = [[ cos(theta(i)),-lambdai(i)*sin(theta(i)), mui(i)*sin(theta(i))];...
               [ sin(theta(i)), lambdai(i)*cos(theta(i)),-mui(i)*cos(theta(i))];...
               [             0,                   mui(i),           lambdai(i)]];
    end

    %------------------------
    % Fonction P(i) : Permet de construire le vecteur position ai
    % (Voir Eq.:4.3b et Eq.:4.7 p.139)
    function val = ai(i)
        val = [[ a(i)*cos(theta(i))];...
               [ a(i)*sin(theta(i))];...
               [ b(i)]];
    end

    %------------------------
    % Fonction PP(i) : Permet de construire la matrice Pi(i) pour le calcul
    % de la matrice Jacobienne. (Voir p.176)
    function val = Pi(i)
       if     i == 1
           val = Qi(1);
       elseif i >  1
           val = Pi(i-1)*Qi(i); % Rcursif
       end
    end

    %------------------------
    % Fonction r(i) : Permet de construire le vecteur ri pour le calcul
    % de la matrice Jacobienne (Voir p.178)
    function val = ri(i)
       if     i == length(theta)
           val = ai(i);
       elseif i <  length(theta)
           val = ai(i)+Qi(i)*ri(i+1);  % Rcursif
       end
    end

    %------------------------
    % Fonction e(i) : Permet de construire le vecteur ei pour le calcul
    % de la matrice Jacobienne (Voir p.175)
    function val = ei(i)
       if i == 1
           val = [0,0,1]'; % par dfinition : [e_1]_1 = [0,0,1]'
       else
           val = Pi(i-1)*[0,0,1]'; % [e_i]_1 est la dernire colonne de P_{i-1}
       end
    end

    %------------------------
    % Fonction e_r(i) : Permet de construire le produit vectoriel de ei(i) 
    % et ri(i) rammen dans le repre 1 (Voir Eq.:5.34 p.179)
    % sachant que [e_i]_i = [0,0,1]'
    function val = ei_ri(i)
       if i == 1
           val = cross([0,0,1]',ri(i));
       else
           val = Pi(i-1)*(  cross([0,0,1]',ri(i))  ); 
       end
    end

    %------------------------
    % Fonction T(i) : Coordonnes du joint i dans le repre de la base
    % (Voir EQ.:4.10 p.140)
    function val = T(i)
       if     i == 1
           val = Ti(1);
       elseif i >  1
           val = T(i-1)*Ti(i); % Rcursif
       end
    end

    %------------------------
    % Fonction Ti(i) : Permet de construire la transforme ti
    % (Voir p.140)
    function val = Ti(i)
        val = [[Qi(i),ai(i)];[0,0,0,1]];
    end





end

% Programme crit par : Benoit Rousseau
% 
% Rfrence: 
%   Fundamentals of Robotic Mechanical Systems
%   Theory, Methods, and Algorithms;
%   Series: Mechanical Engineering Series;
%   Angeles, Jorge; 3rd ed., 2007, XXIV, 549 p., Hardcover; 
%   ISBN: 978-0-387-29412-4
